
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity basic_top is
    generic(
        N           : integer := 8    -- number of bit of PWM counter
    );
    port (
        --- input ports
        i_clk           : in  std_logic;
        i_rstb          : in  std_logic;

        i_enable        : in std_logic;
        o_led           : out std_logic;
		  mosi_top			: out std_logic;
		  sclk_top 			: out std_logic;
		  -- rx_top 			: in std_logic;
		  miso_top 		   : in std_logic;
		  ss_n_top 			: out std_logic;
		  
		  o_led_status		: out std_logic;
		  o_led_busy		: out std_logic
		  
    );
end basic_top;

architecture arch of basic_top is

    signal rst_n : std_logic;

begin

	 rst_n <= not i_rstb;
	 o_led_status <= not  i_rstb;

    pwm_mod : entity work.pwm_led
        generic map(
            N           =>  8    -- number of bit of PWM counter
        )
        port map(
            i_clk           => i_clk,
            i_rstb          => i_rstb,
            i_sync_reset    => '1',
            i_pwm_module    => "11111111",
            i_pwm_width     => "01111111",
            o_pwm           => o_led
        );
		  
    spi_master : entity work.spi_master
           generic map(
                 data_length      => 16)       -- number of bit of PWM counter
           
			  port map(
						clk     				=> i_clk,
						reset_n  			=> rst_n,
						enable  				=>	i_enable,
						cpol    				=>	'0',
						cpha    				=>	'1',
						miso    				=> miso_top,
						sclk    				=> sclk_top,
						ss_n    				=> ss_n_top,
						mosi    				=> mosi_top,
						busy    				=>	o_led_busy,
						tx						=>	"0000010000001100",
						rx	   				=> open 
						);
        
end architecture;
